@@ -8,16 +8,15 @@ from hashlib import md5
 from decimal import Decimal
 from collections import deque, defaultdict
 from itertools import count, islice
-from .six.moves import map
-from .six import b, PY2, integer_types, next, PRE_26, text_type, u, binary_type
-from sys import exc_info
+from six.moves import map
+from six import b, PY2, integer_types, next, text_type, u, binary_type
 from uuid import UUID
 from copy import deepcopy
 from calendar import timegm
-import os
 from distutils.version import LooseVersion
 from struct import Struct
 import time
+import pg8000
 
 # Copyright (c) 2007-2009, Mathieu Fenniak
 # All rights reserved.
@@ -73,9 +72,9 @@ utc = UTC()
 
 
 class Interval(object):
-    """An Interval represents a measurement of time.  In PostgreSQL, an interval
-    is defined in the measure of months, days, and microseconds; as such, the
-    pg8000 interval type represents the same information.
+    """An Interval represents a measurement of time.  In PostgreSQL, an
+    interval is defined in the measure of months, days, and microseconds; as
+    such, the pg8000 interval type represents the same information.
 
     Note that values of the :attr:`microseconds`, :attr:`days` and
     :attr:`months` properties are independently measured and cannot be
@@ -411,9 +410,6 @@ def Binary(value):
     else:
         return value
 
-if PRE_26:
-    bytearray = list
-
 if PY2:
     BINARY = Bytea
 else:
@@ -608,13 +604,13 @@ def timestamp_recv_integer(data, offset, length):
     micros = q_unpack(data, offset)[0]
     try:
         return EPOCH + timedelta(microseconds=micros)
-    except OverflowError:
+    except OverflowError as e:
         if micros == INFINITY_MICROSECONDS:
             return datetime.datetime.max
         elif micros == MINUS_INFINITY_MICROSECONDS:
             return datetime.datetime.min
         else:
-            raise exc_info()[1]
+            raise e
 
 
 # data is double-precision float representing seconds since 2000-01-01
@@ -636,7 +632,7 @@ def timestamp_send_integer(v):
 
 # data is double-precision float representing seconds since 2000-01-01
 def timestamp_send_float(v):
-    return d_pack(timegm(v.timetuple) + v.microsecond / 1e6 - EPOCH_SECONDS)
+    return d_pack(timegm(v.timetuple()) + v.microsecond / 1e6 - EPOCH_SECONDS)
 
 
 def timestamptz_send_integer(v):
@@ -662,13 +658,13 @@ def timestamptz_recv_integer(data, offset, length):
     micros = q_unpack(data, offset)[0]
     try:
         return EPOCH_TZ + timedelta(microseconds=micros)
-    except OverflowError:
+    except OverflowError as e:
         if micros == INFINITY_MICROSECONDS:
             return DATETIME_MAX_TZ
         elif micros == MINUS_INFINITY_MICROSECONDS:
             return DATETIME_MIN_TZ
         else:
-            raise exc_info()[1]
+            raise e
 
 
 def timestamptz_recv_float(data, offset, length):
@@ -902,21 +898,19 @@ class Cursor():
             .. versionadded:: 1.9.11
         """
         try:
-            self._c._lock.acquire()
-            self.stream = stream
+            with self._c._lock:
+                self.stream = stream
 
-            if not self._c.in_transaction and not self._c.autocommit:
-                self._c.execute(self, "begin transaction", None)
-            self._c.execute(self, operation, args)
-        except AttributeError:
+                if not self._c.in_transaction and not self._c.autocommit:
+                    self._c.execute(self, "begin transaction", None)
+                self._c.execute(self, operation, args)
+        except AttributeError as e:
             if self._c is None:
                 raise InterfaceError("Cursor closed")
             elif self._c._sock is None:
                 raise InterfaceError("connection is closed")
             else:
-                raise exc_info()[1]
-        finally:
-            self._c._lock.release()
+                raise e
 
     def executemany(self, operation, param_sets):
         """Prepare a database operation, and then execute it against all
@@ -1027,28 +1021,26 @@ class Cursor():
         pass
 
     def __next__(self):
-        try:
-            self._c._lock.acquire()
-            return self._cached_rows.popleft()
-        except IndexError:
-            if self.portal_suspended:
-                self._c.send_EXECUTE(self)
-                self._c._write(SYNC_MSG)
-                self._c._flush()
-                self._c.handle_messages(self)
-                if not self.portal_suspended:
-                    self._c.close_portal(self)
+        with self._c._lock:
             try:
                 return self._cached_rows.popleft()
             except IndexError:
-                if self.ps is None:
-                    raise ProgrammingError("A query hasn't been issued.")
-                elif len(self.ps['row_desc']) == 0:
-                    raise ProgrammingError("no result set")
-                else:
-                    raise StopIteration()
-        finally:
-            self._c._lock.release()
+                if self.portal_suspended:
+                    self._c.send_EXECUTE(self)
+                    self._c._write(SYNC_MSG)
+                    self._c._flush()
+                    self._c.handle_messages(self)
+                    if not self.portal_suspended:
+                        self._c.close_portal(self)
+                try:
+                    return self._cached_rows.popleft()
+                except IndexError:
+                    if self.ps is None:
+                        raise ProgrammingError("A query hasn't been issued.")
+                    elif len(self.ps['row_desc']) == 0:
+                        raise ProgrammingError("no result set")
+                    else:
+                        raise StopIteration()
 
 if PY2:
     Cursor.next = Cursor.__next__
@@ -1235,23 +1227,19 @@ class Connection(object):
         self._lock = threading.Lock()
 
         if user is None:
-            try:
-                self.user = os.environ['PGUSER']
-            except KeyError:
-                try:
-                    self.user = os.environ['USER']
-                except KeyError:
-                    raise InterfaceError(
-                        "The 'user' connection parameter was omitted, and "
-                        "neither the PGUSER or USER environment variables "
-                        "were set.")
+            raise InterfaceError(
+                "The 'user' connection parameter cannot be None")
+
+        if isinstance(user, text_type):
+            self.user = user.encode('utf8')
         else:
             self.user = user
 
-        if isinstance(self.user, text_type):
-            self.user = self.user.encode('utf8')
+        if isinstance(password, text_type):
+            self.password = password.encode('utf8')
+        else:
+            self.password = password
 
-        self.password = password
         self.autocommit = False
         self._xid = None
 
@@ -1280,35 +1268,29 @@ class Connection(object):
                 self._usock.connect(unix_sock)
 
             if ssl:
-                try:
-                    self._lock.acquire()
-                    import ssl as sslmodule
-                    # Int32(8) - Message length, including self.
-                    # Int32(80877103) - The SSL request code.
-                    self._usock.sendall(ii_pack(8, 80877103))
-                    resp = self._usock.recv(1)
-                    if resp == b('S'):
-                        self._usock = sslmodule.wrap_socket(self._usock)
-                    else:
-                        raise InterfaceError("Server refuses SSL")
-                except ImportError:
-                    raise InterfaceError(
-                        "SSL required but ssl module not available in "
-                        "this python installation")
-                finally:
-                    self._lock.release()
+                with self._lock:
+                    try:
+                        import ssl as sslmodule
+                        # Int32(8) - Message length, including self.
+                        # Int32(80877103) - The SSL request code.
+                        self._usock.sendall(ii_pack(8, 80877103))
+                        resp = self._usock.recv(1)
+                        if resp == b('S'):
+                            self._usock = sslmodule.wrap_socket(self._usock)
+                        else:
+                            raise InterfaceError("Server refuses SSL")
+                    except ImportError:
+                        raise InterfaceError(
+                            "SSL required but ssl module not available in "
+                            "this python installation")
 
             self._sock = self._usock.makefile(mode="rwb")
-        except socket.error:
+        except socket.error as e:
             self._usock.close()
-            raise InterfaceError("communication error", exc_info()[1])
+            raise InterfaceError("communication error", e)
         self._flush = self._sock.flush
         self._read = self._sock.read
-
-        if PRE_26:
-            self._write = self._sock.writelines
-        else:
-            self._write = self._sock.write
+        self._write = self._sock.write
         self._backend_key_data = None
 
         ##
@@ -1517,7 +1499,6 @@ class Connection(object):
             bool: (16, FC_BINARY, bool_send),
             int: (705, FC_TEXT, unknown_out),
             float: (701, FC_BINARY, d_pack),  # float8
-            str: (705, FC_TEXT, text_out),  # unknown
             datetime.date: (1082, FC_TEXT, date_out),  # date
             datetime.time: (1083, FC_TEXT, time_out),  # time
             1114: (1114, FC_BINARY, timestamp_send_integer),  # timestamp
@@ -1538,10 +1519,12 @@ class Connection(object):
         if PY2:
             self.py_types[Bytea] = (17, FC_BINARY, bytea_send)  # bytea
             self.py_types[text_type] = (705, FC_TEXT, text_out)  # unknown
+            self.py_types[str] = (705, FC_TEXT, bytea_send)  # unknown
 
             self.py_types[long] = (705, FC_TEXT, unknown_out)  # noqa
         else:
             self.py_types[bytes] = (17, FC_BINARY, bytea_send)  # bytea
+            self.py_types[str] = (705, FC_TEXT, text_out)  # unknown
 
         try:
             from ipaddress import (
@@ -1608,19 +1591,20 @@ class Connection(object):
         self._flush()
 
         self._cursor = self.cursor()
-        try:
-            self._lock.acquire()
-            code = self.error = None
-            while code not in (READY_FOR_QUERY, ERROR_RESPONSE):
-                code, data_len = ci_unpack(self._read(5))
-                self.message_types[code](self._read(data_len - 4), None)
-            if self.error is not None:
-                raise self.error
-        except:
-            self._close()
-            raise
-        finally:
-            self._lock.release()
+        with self._lock:
+            try:
+                code = self.error = None
+                while code not in (READY_FOR_QUERY, ERROR_RESPONSE):
+                    code, data_len = ci_unpack(self._read(5))
+                    self.message_types[code](self._read(data_len - 4), None)
+                if self.error is not None:
+                    raise self.error
+            except Exception as e:
+                try:
+                    self._close()
+                except Exception:
+                    pass
+                raise e
 
         self.in_transaction = False
         self.notifies = []
@@ -1731,11 +1715,8 @@ class Connection(object):
         # additional_info = data[idx:idx + null]
 
         # psycopg2 compatible notification interface
-        try:
-            self.notifies_lock.acquire()
+        with self.notifies_lock:
             self.notifies.append((backend_pid, condition))
-        finally:
-            self.notifies_lock.release()
 
     def cursor(self):
         """Creates a :class:`Cursor` object bound to this
@@ -1752,11 +1733,8 @@ class Connection(object):
         This function is part of the `DBAPI 2.0 specification
         <http://www.python.org/dev/peps/pep-0249/>`_.
         """
-        try:
-            self._lock.acquire()
+        with self._lock:
             self.execute(self._cursor, "commit", None)
-        finally:
-            self._lock.release()
 
     def rollback(self):
         """Rolls back the current database transaction.
@@ -1764,11 +1742,8 @@ class Connection(object):
         This function is part of the `DBAPI 2.0 specification
         <http://www.python.org/dev/peps/pep-0249/>`_.
         """
-        try:
-            self._lock.acquire()
+        with self._lock:
             self.execute(self._cursor, "rollback", None)
-        finally:
-            self._lock.release()
 
     def _close(self):
         try:
@@ -1781,8 +1756,8 @@ class Connection(object):
             raise InterfaceError("connection is closed")
         except ValueError:
             raise InterfaceError("connection is closed")
-        except socket.error:
-            raise OperationalError(str(exc_info()[1]))
+        except socket.error as e:
+            raise OperationalError(str(e))
         finally:
             self._usock.close()
             self._sock = None
@@ -1793,11 +1768,8 @@ class Connection(object):
         This function is part of the `DBAPI 2.0 specification
         <http://www.python.org/dev/peps/pep-0249/>`_.
         """
-        try:
-            self._lock.acquire()
+        with self._lock:
             self._close()
-        finally:
-            self._lock.release()
 
     def handle_AUTHENTICATION_REQUEST(self, data, cursor):
         assert self._lock.locked()
@@ -1806,7 +1778,7 @@ class Connection(object):
         #               0 = AuthenticationOk
         #               5 = MD5 pwd
         #               2 = Kerberos v5 (not supported by pg8000)
-        #               3 = Cleartext pwd (not supported by pg8000)
+        #               3 = Cleartext pwd
         #               4 = crypt() pwd (not supported by pg8000)
         #               6 = SCM credential (not supported by pg8000)
         #               7 = GSSAPI (not supported by pg8000)
@@ -1823,8 +1795,7 @@ class Connection(object):
                 raise InterfaceError(
                     "server requesting password authentication, but no "
                     "password was provided")
-            self._send_message(
-                PASSWORD, self.password.encode("ascii") + NULL_BYTE)
+            self._send_message(PASSWORD, self.password + NULL_BYTE)
             self._flush()
         elif auth_code == 5:
             ##
@@ -1840,8 +1811,8 @@ class Connection(object):
                     "server requesting MD5 password authentication, but no "
                     "password was provided")
             pwd = b("md5") + md5(
-                md5(self.password.encode("ascii") + self.user).
-                hexdigest().encode("ascii") + salt).hexdigest().encode("ascii")
+                md5(self.password + self.user).hexdigest().encode("ascii") +
+                salt).hexdigest().encode("ascii")
             # Byte1('p') - Identifies the message as a password message.
             # Int32 - Message length including self.
             # String - The password.  Password may be encrypted.
@@ -1879,11 +1850,10 @@ class Connection(object):
             except KeyError:
                 try:
                     params.append(self.inspect_funcs[typ](value))
-                except KeyError:
+                except KeyError as e:
                     raise NotSupportedError(
-                        "type " + str(exc_info()[1]) +
-                        "not mapped to pg type")
-        return params
+                        "type " + str(e) + "not mapped to pg type")
+        return tuple(params)
 
     def handle_ROW_DESCRIPTION(self, data, cursor):
         count = h_unpack(data)[0]
@@ -1904,7 +1874,7 @@ class Connection(object):
     def execute(self, cursor, operation, vals):
         if vals is None:
             vals = ()
-        from . import paramstyle
+        paramstyle = pg8000.paramstyle
         cache = self._caches[paramstyle]
 
         try:
@@ -1915,8 +1885,7 @@ class Connection(object):
 
         args = make_args(vals)
         params = self.make_params(args)
-
-        key = tuple(oid for oid, x, y in params), operation
+        key = operation, params
 
         try:
             ps = cache['ps'][key]
@@ -1960,11 +1929,13 @@ class Connection(object):
 
             try:
                 self._flush()
-            except AttributeError:
+            except AttributeError as e:
                 if self._sock is None:
                     raise InterfaceError("connection is closed")
                 else:
-                    raise exc_info()[1]
+                    raise e
+            except socket.error as e:
+                raise OperationalError(str(e))
 
             self.handle_messages(cursor)
 
@@ -2054,11 +2025,11 @@ class Connection(object):
             self._write(i_pack(len(data) + 4))
             self._write(data)
             self._write(FLUSH_MSG)
-        except ValueError:
-            if str(exc_info()[1]) == "write to closed file":
+        except ValueError as e:
+            if str(e) == "write to closed file":
                 raise InterfaceError("connection is closed")
             else:
-                raise exc_info()[1]
+                raise e
         except AttributeError:
             raise InterfaceError("connection is closed")
 
